//  CANRx.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>

#include "flexcansdk.h"
#include <gpiosdk.h>

#define CAN_PORT 2
#define CAN_BITRATE         (125*1000)  // 125 kHz
#define CAN_READ_INTERVAL   1000        // Set to INFINITE to wait forever for a CAN Frame

// GPIO Variables
GPIO Button2;
GPIO LedG1;

DWORD WINAPI CANReadThread(LPVOID lpParameter)
{
	HANDLE hFinishEvent = (HANDLE)lpParameter;
	CAN_FRAME canFrame = { 0 };
	canFrame.bExtended = CAN_EXTENDED;  // Receive any extended frame
	do
	{
		if (ERROR_SUCCESS == FlexCANReadFrame(CAN_PORT, &canFrame, CAN_READ_INTERVAL, 0))
		{
			// Set LED1 based on data[0]
			LedG1.Value = canFrame.btData[0];
			GpioWritePin(&LedG1);
		}
	} while (WAIT_TIMEOUT == WaitForSingleObject(hFinishEvent, 0));

	return 0;
}

void SetupGpio(GPIO* pGpio, GPIO_PORT port, GPIO_PIN pin, GPIO_DIR direction, UINT32 initialValue)
{
	GPIO_CONFIG GpioConfig;
	GpioConfig.Port = port;
	GpioConfig.Pin = pin;
	GpioGetConfig(&GpioConfig);

	GpioConfig.Pull = GPIO_PULL_UP_22K;
	GpioConfig.Drive = GPIO_DRIVE_40_OHM;
	GpioConfig.Direction = direction;
	GpioSetConfig(&GpioConfig);

	pGpio->Port = port;
	pGpio->Pin = pin;
	pGpio->Value = initialValue;
	GpioWritePin(pGpio);
}

int wmain(int argc, wchar_t *argv[])
{
	HANDLE hFinishEvent = NULL;
	HANDLE hReadThread = NULL;

	// Initialise CAN and set the data rate
	if (!FlexCANInit(CAN_PORT))
		return 0;

	DWORD dwBitrate = CAN_BITRATE;
	if (!FlexCANSetBitrate(CAN_PORT, &dwBitrate))
		return 0;

	// Initialise the GPIO
	if (!GpioInit())
		return 0;

	// Setup GPIO for the Button (to terminate the app) and LED to show status
	SetupGpio(&Button2, GPIO_PORT_3, GPIO_PIN_30, GPIO_DIR_IN, 0);
	SetupGpio(&LedG1, GPIO_PORT_3, GPIO_PIN_5, GPIO_DIR_OUT, 0);

	// Create the event that will be used to signal the application to terminate
	hFinishEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	// Create the read thread that will receive all CAN Frames
	hReadThread = CreateThread(NULL, 0, CANReadThread, (LPVOID)hFinishEvent, 0, NULL);

	// Wait for Button2 to be pressed to terminate the application
	do
	{
		GpioReadPin(&Button2);
		if (Button2.Value == 0)
			SetEvent(hFinishEvent);
	} while (WAIT_TIMEOUT == WaitForSingleObject(hFinishEvent, 0));

	// Cleanup
	FlexCANDeinit(CAN_PORT);
	GpioDeinit();

	if (hFinishEvent)
		CloseHandle(hFinishEvent);

	return 0;
}
