//  CANTx.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>

#include "flexcansdk.h"
#include <gpiosdk.h>

#define CAN_PORT 1
#define CAN_BITRATE         (125*1000)  // 125 kHz
#define CAN_READ_INTERVAL   1000        // Set to INFINITE to wait forever for a CAN Frame

void SetupGpio(GPIO* pGpio, GPIO_PORT port, GPIO_PIN pin, GPIO_DIR direction, UINT32 initialValue)
{
	GPIO_CONFIG GpioConfig;
	GpioConfig.Port = port;
	GpioConfig.Pin = pin;
	GpioSetConfig(&GpioConfig);

	GpioConfig.Pull = GPIO_PULL_UP_22K;
	GpioConfig.Drive = GPIO_DRIVE_40_OHM;
	GpioConfig.Direction = direction;
	GpioSetConfig(&GpioConfig);

	pGpio->Port = port;
	pGpio->Pin = pin;
	pGpio->Value = initialValue;
	GpioWritePin(pGpio);
}

int wmain(int argc, wchar_t *argv[])
{
	BOOL bTerminate = FALSE;
	GPIO Button1;
	GPIO Button2;

	// Initialise CAN and set the data rate
	if (!FlexCANInit(CAN_PORT))
		return 0;

	DWORD dwBitrate = CAN_BITRATE;
	if (!FlexCANSetBitrate(CAN_PORT, &dwBitrate))
		return 0;

	// Initialise the GPIO
	if (!GpioInit())
		return 0;

	// Setup the Button GPIOs
	SetupGpio(&Button1, GPIO_PORT_3, GPIO_PIN_31, GPIO_DIR_IN, 0);
	SetupGpio(&Button2, GPIO_PORT_3, GPIO_PIN_30, GPIO_DIR_IN, 0);

	while (!bTerminate)
	{
		// Read button 1 
		GpioReadPin(&Button1);

		// Transmit CAN frame with button status
		CAN_FRAME canFrame = { 0 };
		canFrame.bExtended = TRUE;
		canFrame.btData[0] = !Button1.Value; // intert so 1 indicated button pressed
		canFrame.wDataLen = 1;
		canFrame.dwID = 0xE080001;
		FlexCANWriteFrame(CAN_PORT, FALSE, 0, &canFrame, 500);

		// if button 2 is pressed (0) then exit
		GpioReadPin(&Button2);
		if (Button2.Value == 0)
			bTerminate = TRUE;

		Sleep(100);
	}

	// Cleanup
	FlexCANDeinit(CAN_PORT);
	GpioDeinit();

	return 0;
}
